package com.sleepycat.je.log;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.EnvironmentImpl;

public class SynchronizedIOFileManager extends IOFileManager {
//TODO who does this match with IOFileManager....aspect
	static aspect SynchronizedIOAspect {

		IOFileManager around(EnvironmentImpl envImpl, File dbEnvHome,
				boolean readOnly) throws DatabaseException :
			call(IOFileManager.new(EnvironmentImpl, File, boolean)) && args(envImpl, dbEnvHome, readOnly){

			return new SynchronizedIOFileManager(envImpl, dbEnvHome, readOnly);
		}
	}

	public SynchronizedIOFileManager(EnvironmentImpl envImpl, File dbEnvHome,
			boolean readOnly) throws DatabaseException {
		
		super(envImpl, dbEnvHome, readOnly);
	}

	void readFromFile(RandomAccessFile file, ByteBuffer readBuffer, long offset)
			throws IOException {

		/*
		 * Synchronization on the file object is needed
		 * because two threads may call seek() on the same file object.
		 */
		synchronized (file) {
			super.readFromFile(file,readBuffer,offset);
		}
		
	}

	protected int writeToFile(RandomAccessFile file, ByteBuffer data,
			long destOffset) throws IOException, DatabaseException {

		int totalBytesWritten = 0;
		/*
		 * Synchronization on the file object is needed
		 * because two threads may call seek() on the same file object.
		 */
		synchronized (file) {
			super.writeToFile(file,data,destOffset);
		}
	
	return totalBytesWritten;
	}
}
